package board;

import com.exln.dxe.*;
import com.exln.dxe.client.*;
import com.exln.dxe.filesystem.*;
import org.w3c.dom.*;

import java.io.*;
import java.net.*;

public class Board {
	private static String xmlResource = "bbs:/board/board.xml";
	private static String parentPath = "/articles";
	private static String currentPath = "/articles/article";
	
	public static BoardRec[] list(int start, int end, String searchWord) {
		String searchCondition = 
			( searchWord == null ? "" : "[contains( title,'"+searchWord+"')]");
		String pageCondition = "[position() >="+ start +" and position() <"+ end +"]";
		String query = currentPath + searchCondition + pageCondition;
		
		System.out.println(query);
		Session session = XlnClientSessionFactory.getSession();
		XPath xpath = session.createXPath(query);	
		int count = xpath.execute(xmlResource);
		
		NodeList nodes = xpath.getResult().getNodeList();
		int loopNumber = nodes.getLength();
		BoardRec[] recs = new BoardRec[loopNumber];
		
		for( int i = 0 ; i < loopNumber ; i++ ) {
			Element element = (Element)nodes.item(i);
			String idTime = element.getAttributes().getNamedItem("idTime").getFirstChild().getNodeValue();
			String title = element.getElementsByTagName("title").item(0).getFirstChild().getNodeValue();			
			String writer = element.getElementsByTagName("writer").item(0).getFirstChild().getNodeValue();
			String email = element.getElementsByTagName("email").item(0).getFirstChild().getNodeValue();
			recs[i] = new BoardRec();
			recs[i].setTime(idTime);
			recs[i].setTitle(title);
			recs[i].setWriter(writer);
			recs[i].setEmail(email);
		}
		
		return recs;
	}
	
	public static int getArticleLength(String searchWord) {		
		String searchCondition = 
			( searchWord == null ? "" : "[contains( title,'"+searchWord+"')]");
		String query = "count(" + currentPath + searchCondition + ")";		
		
		Session session = XlnClientSessionFactory.getSession();
		XPath xpath = session.createXPath(query);
		int count = xpath.execute(xmlResource);		
		int number = (int)xpath.getResult().getValueAsNumber();
		return number;
	}
	
	public static BoardRec view(BoardRec recParam) {		
		String query = currentPath + "[@idTime='" + recParam.getTime() + 
			"' and email='"+ recParam.getEmail() +"']";
		
		Session session = XlnClientSessionFactory.getSession();
		XPath xpath = session.createXPath(query);	
		int count = xpath.execute(xmlResource);
		
		NodeList nodes = xpath.getResult().getNodeList();
		int loopNumber = nodes.getLength();		
		BoardRec rec = new BoardRec();
		if( loopNumber > 0 ) {
			Element element = (Element)nodes.item(0);
			String title = 
				element.getElementsByTagName("title").item(0).getFirstChild().getNodeValue();
			String body = 
				element.getElementsByTagName("body").item(0).getFirstChild().getNodeValue();			
			String writer = 
				element.getElementsByTagName("writer").item(0).getFirstChild().getNodeValue();			
			rec.setTitle(title);
			rec.setBody(body);
			rec.setWriter(writer);
			rec.setTime( recParam.getTime() );
			rec.setEmail( recParam.getEmail() );
		} 
		return rec;
	}
	
	public static void insert(BoardRec rec) {
		String query = parentPath;		
		String locationOption = "firstchild";
		String insertXML = 
			"<?xml version='1.0'?>" +
			"<xlnupdate version='1.0'>" +
			"<update select='" + query + "'>" +
			"<element location=\"" + locationOption + "\">" +
			"<article idTime='" + rec.getTime() + "'>" +
			"<title>" + rec.getTitle() + "</title>" +
			"<body>" + rec.getBody() + "</body>" +
			"<writer>" + rec.getWriter() + "</writer>" + 
			"<email>" + rec.getEmail() + "</email>" +
			"</article>" +			
			"</element>" +
			"</update>" +
			"</xlnupdate>";			
		
		Session session = XlnClientSessionFactory.getSession();
		Update insert = session.createUpdate(insertXML);
		int count = insert.execute(xmlResource);
	}
	
	public static void update(BoardRec rec) {		
		String query = currentPath;
		query += "[@idTime='" + rec.getTime() + 
			"' and email='" + rec.getEmail() + "']/body/text()";
		String updateXML = 
			"<?xml version='1.0'?>" +
			"<xlnupdate version='1.0'>" +
			"<update select=\"" + query + "\">" +
			"<text>" + rec.getBody() + "</text>" +
			"</update>" +
			"</xlnupdate>";			
		System.out.println(updateXML);
		Session session = XlnClientSessionFactory.getSession();
		Update update = session.createUpdate(updateXML);
		int count = update.execute(xmlResource);
	}
	
	public static void delete(BoardRec rec) {		
		String query = currentPath;
		query += "[@idTime='"+rec.getTime()+"' and email='"+rec.getEmail()+"']";
		String deleteXML = 			
			"<?xml version='1.0'?>" +
			"<xlnupdate version='1.0'>" +
			"<remove select=\"" + query + "\"/>" +
			"</xlnupdate>";
		System.out.println(deleteXML);		
		Session session = XlnClientSessionFactory.getSession();
		Update delete = session.createUpdate(deleteXML);
		int count = delete.execute(xmlResource);
	}
	
	private static boolean exist(BoardRec rec) {
		String query = currentPath;
		query += "[@idTime='" + rec.getTime() + 
			"' and email='" + rec.getEmail() + "']";
		
		Session session = XlnClientSessionFactory.getSession();
		XPath xpath = session.createXPath(query);	
		int count = xpath.execute(xmlResource);
		int number = (int)xpath.getResult().getValueAsNumber();
		
		return ( number > 0 ? true : false );
	}
}