import org.w3c.dom.*;
import org.xml.sax.*;
import org.apache.xerces.parsers.*;

public class ValidChecker implements ErrorHandler {
	String           file;
	InputSource  input;
	boolean      valid;
	
	public ValidChecker(String file) {
		this.file = file;
	}
	
	public ValidChecker(InputSource input) {
		this.input = input;
	}
	
	public void  validate() {
		valid = true;		
		try {
			DOMParser parser = new DOMParser();
			parser.setFeature(
				"http://xml.org/sax/features/namespaces", true);
			parser.setFeature(
				"http://xml.org/sax/features/validation", true);			
			parser.setFeature(
				"http://apache.org/xml/features/validation/schema", true);
			parser.setErrorHandler(this);
			if(file != null) {
				parser.parse(file);
			} else {
				parser.parse(input);
			}
		} catch(Exception e) {
			valid = false;
			e.printStackTrace();
		}
	}
	
	public boolean isValid() {
		return valid;
	}
	
	public void warning(SAXParseException e) throws SAXException {
		System.out.print(" ޽");
		System.out.print(e.getLineNumber() + " : ");
		System.out.println(e.getMessage());
	}
	
	public void error(SAXParseException e) throws SAXException {
		valid = false;
		System.out.print("ȿ  :");	
		System.out.print(e.getLineNumber() + " : ");		
		System.out.println(e.getMessage());		
	}
	 
	public void fatalError(SAXParseException e) throws SAXException {
		valid = false;
		System.out.print("Well-formed  :"); 
		System.out.print(e.getLineNumber() + " : ");
		System.out.println(e.getMessage());		
	}
	
	public static void main(String args[]) {
		if(args.length == 0) {
			System.out.println("usage: java ValidChecker <xml-file>");
			System.exit(0);
		}		
		ValidChecker checker = 	new ValidChecker(args[0]);
		checker.validate();
		if(checker.isValid()) {
			System.out.println(args[0] + " ȿ XML Դϴ.");
		} 
	}
}
